import argparse


class PrettyHelpFormatter(
    argparse.RawTextHelpFormatter, argparse.ArgumentDefaultsHelpFormatter
):
    def _format_action(self, action: argparse.Action) -> str:
        return super()._format_action(action) + "\n"



import argparse
import os
import sys
from pathlib import Path
from typing import Any, Dict, Optional

import pandas

from convpandas.common.cli import PrettyHelpFormatter


def _read_excel(xlsx_file: str, sheet_name: Optional[str]) -> pandas.DataFrame:
    read_excel_kwargs: Dict[str, Any] = {"header": None}
    if sheet_name is not None:
        read_excel_kwargs.update({"sheet_name": sheet_name})
    return pandas.read_excel(xlsx_file, **read_excel_kwargs)


def _to_csv(
    df: pandas.DataFrame,
    filepath_or_buffer: Any,
    sep: str,
    encoding: str,
    quotechar: str,
) -> None:
    if isinstance(filepath_or_buffer, str):
        Path(filepath_or_buffer).parent.mkdir(exist_ok=True, parents=True)

    to_csv_kwargs = {
        "sep": sep,
        "encoding": encoding,
        "quotechar": quotechar,
        "header": False,
        "index": False,
    }
    df.to_csv(filepath_or_buffer, **to_csv_kwargs)


def xlsx2csv(
    xlsx_file: str,
    csv_file: str,
    sheet_name: Optional[str],
    sep: str,
    encoding: str,
    quotechar: str,
):
    if not os.path.exists(xlsx_file):
        print(f"No such file or directory: '{xlsx_file}'", file=sys.stderr)
        sys.exit(1)

    df = _read_excel(xlsx_file, sheet_name=sheet_name)

    filepath_or_buffer: Any = csv_file
    if csv_file == "-":
        filepath_or_buffer = sys.stdout

    _to_csv(df, filepath_or_buffer, sep=sep, encoding=encoding, quotechar=quotechar)


def main(args):
    xlsx2csv(
        xlsx_file=args.xlsx_file,
        csv_file=args.csv_file,
        sheet_name=args.sheet_name,
        sep=args.sep,
        encoding=args.encoding,
        quotechar=args.quotechar,
    )


def add_parser(subparsers: argparse._SubParsersAction):
    parser = subparsers.add_parser(
        "xlsx2csv",
        help="Convert xlsx file to csv file.",
        formatter_class=PrettyHelpFormatter,
    )
    parser.set_defaults(command_help=parser.print_help)

    parser.add_argument("xlsx_file", type=str)
    parser.add_argument("csv_file", type=str)

    parser.add_argument(
        "--sheet_name",
        type=str,
        help="Sheet name when reading xlsx. If not specified, read 1st sheet.",
    )

    parser.add_argument(
        "--sep", default=",", help="Field delimiter for the output file."
    )

    parser.add_argument(
        "--encoding",
        default="utf-8",
        help="A string representing the encoding to use in the output file.",
    )
    parser.add_argument(
        "--quotechar",
        default='"',
        help="Character used to quote fields.",
    )

    parser.set_defaults(subcommand_func=main)
    return parser



import argparse
from typing import Optional, Sequence

import convpandas
from convpandas.command.csv2xlsx import add_parser as csv2xlsx_add_parser
from convpandas.command.xlsx2csv import add_parser as xlsx2csv_add_parser
from convpandas.common.cli import PrettyHelpFormatter


def cli(arguments: Optional[Sequence[str]] = None):
    """
    注意： `deprecated`なツールは、サブコマンド化しない。

    Args:
        arguments: コマンドライン引数。テストコード用

    """

    parser = argparse.ArgumentParser(
        description="Command Line Interface for AnnoFab",
        formatter_class=PrettyHelpFormatter,
    )
    parser.add_argument(
        "--version", action="version", version=f"convpandas {convpandas.__version__}"
    )
    parser.set_defaults(command_help=parser.print_help)

    subparsers = parser.add_subparsers(dest="command_name")
    csv2xlsx_add_parser(subparsers)
    xlsx2csv_add_parser(subparsers)

    if arguments is None:
        args = parser.parse_args()
    else:
        args = parser.parse_args(arguments)

    if hasattr(args, "subcommand_func"):
        args.subcommand_func(args)

    else:
        # 未知のサブコマンドの場合はヘルプを表示
        args.command_help()


if __name__ == "__main__":
    cli()



import argparse
import io
import sys
from pathlib import Path
from typing import Dict, List, Union

import click
import openpyxl
import pandas

from convpandas.common.cli import PrettyHelpFormatter


def _read_csv(
    csv_file: Union[str, io.TextIOBase],
    sep: str,
    encoding: str,
    quotechar: str,
) -> pandas.DataFrame:
    read_csv_kwargs = {
        "sep": sep,
        "encoding": encoding,
        "quotechar": quotechar,
        "header": None,
    }
    return pandas.read_csv(csv_file, **read_csv_kwargs)


def _to_numeric(value):
    return pandas.to_numeric(value, errors="ignore")


def _do_not_anything(value):
    return value


def _to_excel(
    df_dict: Dict[str, pandas.DataFrame],
    xlsx_file: str,
    string_to_numeric: bool = True,
) -> None:
    Path(xlsx_file).parent.mkdir(exist_ok=True, parents=True)
    if string_to_numeric:
        convert_func = _to_numeric
    else:
        convert_func = _do_not_anything

    workbook = openpyxl.Workbook(write_only=True)
    for sheet_name, df in df_dict.items():
        if sheet_name == "-":
            worksheet = workbook.create_sheet()
        else:
            worksheet = workbook.create_sheet(title=sheet_name)
        values = df.values
        for row_index in range(df.shape[0]):
            row = values[row_index]
            worksheet.append([convert_func(value) for value in row])
    workbook.save(xlsx_file)


def append_df_to_dict(
    df_dict: Dict[str, pandas.DataFrame], sheet_name: str, df: pandas.DataFrame
):
    if sheet_name not in df_dict:
        df_dict[sheet_name] = df
        return

    index = 1
    while True:
        new_sheet_name = f"{sheet_name}_{index}"
        if new_sheet_name not in df_dict:
            df_dict[new_sheet_name] = df
            return df_dict
        index += 1


def csv2xlsx(
    csv_files: List[str],
    xlsx_file: str,
    *,
    sep: str,
    encoding: str,
    quotechar: str,
    string_to_numeric: bool,
    sheet_names: List[str],
):
    if sheet_names is not None:
        if len(sheet_names) != len(csv_files):
            print(
                "Size of csv_files and size of sheet_name do not match.",
                file=sys.stderr,
            )
            sys.exit(1)

    df_dict: Dict[str, pandas.DataFrame] = {}

    if len(csv_files) == 1 and csv_files[0] == "-":
        str_stdin = click.get_text_stream("stdin", encoding=encoding).read()
        df = _read_csv(
            io.StringIO(str_stdin), sep=sep, encoding=encoding, quotechar=quotechar
        )
        if sheet_names is None:
            df_dict["-"] = df
        else:
            df_dict[sheet_names[0]] = df
        _to_excel(df_dict, xlsx_file=xlsx_file, string_to_numeric=string_to_numeric)
        return

    if sheet_names is None:
        for file in csv_files:
            file_path = Path(file)
            if not file_path.exists():
                print(f"No such file: '{file}'", file=sys.stderr)
                sys.exit(1)

            df = _read_csv(file, sep=sep, encoding=encoding, quotechar=quotechar)
            append_df_to_dict(df_dict, sheet_name=file_path.stem, df=df)
    else:
        for file, name in zip(csv_files, sheet_names):
            file_path = Path(file)
            if not file_path.exists():
                print(f"No such file: '{file}'", file=sys.stderr)
                sys.exit(1)

            df = _read_csv(file, sep=sep, encoding=encoding, quotechar=quotechar)
            append_df_to_dict(df_dict, sheet_name=name, df=df)

    _to_excel(df_dict, xlsx_file=xlsx_file, string_to_numeric=string_to_numeric)


def main(args):
    csv2xlsx(
        csv_files=args.csv_files,
        xlsx_file=args.xlsx_file,
        sep=args.sep,
        encoding=args.encoding,
        quotechar=args.quotechar,
        string_to_numeric=args.string_to_numeric,
        sheet_names=args.sheet_name,
    )


def add_parser(subparsers: argparse._SubParsersAction):
    parser = subparsers.add_parser(
        "csv2xlsx",
        help="Convert csv file to xlsx file.",
        formatter_class=PrettyHelpFormatter,
    )
    parser.set_defaults(command_help=parser.print_help)

    parser.add_argument("csv_files", type=str, nargs="+")
    parser.add_argument("xlsx_file", type=str)

    parser.add_argument("--sep", default=",", help="Delimiter to use when reading csv.")

    parser.add_argument(
        "--encoding",
        default="utf-8",
        help="Encoding to use when reading csv. List of Python standard encodings.\n"
        "https://docs.python.org/3/library/codecs.html#standard-encodings",
    )
    parser.add_argument(
        "--quotechar",
        default='"',
        help="The character used to denote the start and end of a quoted item when reading csv.",
    )

    parser.add_argument(
        "--string_to_numeric",
        action="store_true",
        help="If specified, convert string to numeric.",
    )

    parser.add_argument("--sheet_name", type=str, nargs="+")

    parser.set_defaults(subcommand_func=main)
    return parser
